SystemOrganization addCategory: #Matcher!
SystemOrganization addCategory: #'Matcher-Test'!

TestCase subclass: #MatcherTest
	instanceVariableNames: 'techTicketApp someTicket someTicketId secondTicketId testTechId billId emptyTechId workerMatchApp someProject someProjectId pepeId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher-Test'!

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 15:37'!
setUp
	self setUpTicketTests.
	self setUpProjectTests.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:34'!
setUpProjectTests
	| newProject worker |
	
	MatcherSkill setWeight: 0.4.
	MatcherLocation setWeight: 0.6.
	
	someProject := Project newWithDescription: 'Some Project' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'Pepe Smith').
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'Coding')."skil"
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'HTML')."skill"
	someProject addRequirement: (workerMatchApp getMatchItemByName: 'ECommerce'). "location"
	someProjectId := workerMatchApp addMatchActivity: someProject.
	
	"Worker Creation"
	worker := Worker withFirstName: 'Pepe' lastName: 'Skillz' address: '0 Worker Way' city: 'Worker Town' state: 'WA' zip: '22333' phone: '5555551414'.
	worker assignMatchItem: (workerMatchApp getMatchItemByName: 'Coding').
	worker assignMatchItem: (workerMatchApp getMatchItemByName: 'HTML').
	pepeId := workerMatchApp addMatchResource: worker.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 21:33'!
setUpTicketTests
	| tech newProject |
	
	MatcherSkill setWeight: 0.7.
	MatcherLocation setWeight: 0.3.
	
	techTicketApp := TechTicketApp default.
	techTicketApp initialize.
	techTicketApp privateClearItems.
	workerMatchApp := WorkerMatchApp default.
	workerMatchApp initialize.
	workerMatchApp privateClearItems.

	tech := (Tech withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Philadelphia').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Washington DC').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Computer Repair').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Testing').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'CCNA').
	tech setAvailability: #(monday wednesday friday) asOrderedCollection.
	
	testTechId := techTicketApp addMatchResource: tech.
	
	tech := (Tech withFirstName: 'Empty' lastName: 'Skills' address: '0 Tech Way' city: 'Tech Town' state: 'WA' zip: '22333' phone: '5555551414').
	emptyTechId := techTicketApp addMatchResource: tech.
	
	tech := (Tech withFirstName: 'Bill' lastName: 'Lumberg' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Orlando').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Computer Repair').
	tech assignMatchItem: (techTicketApp getMatchItemByName: 'Java').
	billId := techTicketApp addMatchResource: tech.
	
	someTicket := Ticket newWithDescription: 'Test Ticket' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'Webmaster').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Testing').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Computer Repair').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'New York City').

	someTicketId := techTicketApp addMatchActivity: (someTicket).
	techTicketApp assignResourceId: billId toActivityId: someTicketId.	
	
	someTicket := Ticket newWithDescription: 'Another Ticket' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'Dispatch').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Testing').
	someTicket addRequirement: (techTicketApp getMatchItemByName: 'Washington DC'). 
	
	secondTicketId := techTicketApp addMatchActivity: (someTicket).
	techTicketApp assignResourceId: emptyTechId toActivityId: secondTicketId.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 22:10'!
testActivityVolumeByHourReprt
	"Creates a dictionary of all hours of the day and provides the number of tickets created in that hour."
	"This report can provide managers with helpful analysis of high ticket traffic times for scheduling, for example."
	"The MatchReport class holds this report since it uses no items specific to any subclasses MatcherActivity."
	| tickVolumeDict |
	tickVolumeDict := MatcherReport reportTicketVolumeByHour: techTicketApp.
	self assert: (tickVolumeDict at: (DateAndTime now hour24 asFloat)) = (techTicketApp activityCount).! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 12:49'!
testFrameworkUsage
| app worker addedTicketNum resultingWorkers |

"First, we instantiate the main application, this concrete class inherits from MatcherMainApp"
"MatcherMainApp provides many generic methods to manage Resources (i.e. a Person)"
"MatcherMainApp provides many generic methods to manage Activities for these Resources (i.e. a Ticket, or a Project)"
"MatcherMainApp helps manage a collection of Items (i.e. skills) which are used as Attributes for these Activties and Resources for mutual matching"

"In this case, we have created a concrete class, WorkerMatchApp, which extends the Super class Initializer, adding a few Attributes relevant to this space"
app := WorkerMatchApp default.


"see setUp for Pepe a Nothing more expected ->nd someProject"
worker := Worker withFirstName: 'Juanito' lastName: 'Augenblick' address: '123 Main Street' city: 'Shiny Town' state: 'TX' zip: '12333' phone: '5555555454'. 
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'HTML').
app addMatchResource: worker.

resultingWorkers := app findResourceMatchForActivity: someProject.

Transcript show: (someProject bestMatchDictionary).  Transcript cr.

self assert: resultingWorkers size = 2.

self assert: (someProject matchWithTopScore) getResource fullName = 'Pepe Skillz'.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 23:53'!
testFullTechTicketInstructions
| techTickApp newTech newTechId newTicket newTicketId newReportDict |

"To begin your application, you will want to subclass the MatcherApp class, which provides you with the base functionality of the framework, and will hold a number of objects (such as Activities and Resources) you will use directly, or indirectly for more advanced functionality.  We have subclasses an app named TechTicketApp to illustrate this.  There is a method 'buildInitialSet' which you can utilize to set up any default MatcherItems (see TechTicketApp -> buildInitialSet for some examples) that you may know you want to reference in your application"

techTickApp := TechTicketApp default.
techTickApp initialize. "Since the TechTicketApp is a singleton, I want to manually clear out the items for the purposes of this test."

"If you checked out the buildInitialSet for TechTicketApp, you will notice a number of subclasses of MatcherItem, including both MatcherLocation and MatcherSkill.  These are examples of the types of requirements you will be assigning to both your Activites and your Resources to find proper matches.  You can subclass MatcherItem and provide any additional criteria that may fit that requirement of your activity.  The MatcherSkill for example, includes an additional item of a 'skill level' which needs to be met or exceeded to be considered equal to (notice the overriden @ call as well).  buildInitialSet can provide your system with a base set to utilize for Activites/Resources as mentioned above.  This particular code created the matcherItem, and adds it to the main application for reference later on by other objects."

techTickApp addMatchItem: (MatcherLocation newWithName: 'Topeka' address: '1 Central St.' city: 'Topeka' state: 'KS' additionalInfo: 'Main KS Office').

"It is worth mentioning here that your MatcherItems include a 'weight' property which you can utilize to put more or less emphasis on a particular type of MatcherItem when having the system decide which resources best match your activites.  For example, we can have our subclass of MatcherLocation be less important, and have a weight of .3 and assign a more important value of .7 to MatcherSkill, because we think we'd rather have the emphasis on someone really good at an activity rather than just very close to it.  You can assign any values to these, they do not have to add up to 1.0 as in traditional statistics, but can do so to keep things simple to read."

MatcherSkill setWeight: 0.7.
MatcherLocation setWeight: 0.3.

"Next we can take a look at 
the resources you will be adding to your system and will be able to match based on your criteria, or the system's default criteria.  You can subclass the base class MatcherResource which hold a series of basic items (name, address, etc) and of course a collection of MatcherItems which we will use to compare them to MatcherActivites.  Subclass the MatcherResource if you have specific instance requirements that are not met by the MatcherItem.  We have added a subclass of Tech which includes one additional item of a currentLocation, so we can use this information to better assign that Tech (a resource) to Tickets (an activity).  After creating the tech, we assign them some skills and locations (subclassed MatcherItems) then add it to the main application who is keeping track of our collections - it returns us the assigned ID of the tech added." 

newTech := (Tech withFirstName: 'John' lastName: 'Smith' address: '1 Begin Way' city: 'Jamestown' state: 'VA' zip: '10101' phone: '2228889999').
newTech assignMatchItem: (techTickApp getMatchItemByName: 'Orlando'). "MatcherLocation"
newTech assignMatchItem: (techTickApp getMatchItemByName: 'Computer Repair'). "MatcherSkill"
newTech assignMatchItem: (techTickApp getMatchItemByName: 'Java'). "MatcherSkill"
newTechId := techTickApp addMatchResource: newTech.
self assert: (techTickApp resourceCount) = 1.

"Now all we need is an activity to which we can assign these resources to.  Again, these can be subclassed from MatcherActivity to include any additional items you may want to run checks on or items you may want to report on regarding your resources or the activites themselves.  Here we are creating a Ticket (Activity), assigning some MatcherItems to match up with the Techs, and then adding it into the system, which returns us the assigned ID."

newTicket := Ticket newWithDescription: 'Example Ticket' priority: 1 dueBy: (Date today) createdBy: (MatcherUser fullName: 'DocTest').
newTicket addRequirement: (techTickApp getMatchItemByName: 'Testing').
newTicket addRequirement: (techTickApp getMatchItemByName: 'Computer Repair').
newTicket addRequirement: (techTickApp getMatchItemByName: 'Philadelphia').
newTicketId := techTickApp addMatchActivity: (newTicket).
self assert: (techTickApp activityCount) = 1.

"The MatcherActivity has methods which will run the hard work in determining how your Activity is matched up with a resource, and the behavior of these methods can be modified using a strategy subclassed from MatcherScoreEngine (assign the strategy to the scoreEngineStrategy instance variable).  The base functionality in MatcherScoreEngine matches up activities based on all criteria found in the requirements collection and takes into account any weights you have assigned to those classes.  If you think this is too limited, take a gander at overriding the function matchScoreForResource:withActivity: in your subclass.  This gives you ultimate power in how your resource assignments can be reported to you.  In our example, we are just assigning it the base class, since the framework's innate functionality will work for matching Techs with Tickets.  Note you do not need to explicitly declare this starategy, as it is given by default, but we are adding it again for illustrative purposes."

(techTickApp getMatchActivityById: newTicketId) scoreEngineStrategy: (MatcherScoreEngine new).
self assert: ((techTickApp findResourceMatchForActivity: (techTickApp getMatchActivityById: newTicketId)) at: newTech) getScore = 41.18.
"John Smith matches with a weighted score of 41.18 out of 100"

"At this point we are also going to go ahead and assign our new tech to our new ticket"

techTickApp assignResourceId: newTechId toActivityId: newTicketId.
(techTickApp getMatchActivityById: newTicketId) assignedResourcesDictionary size = 1.

"To make some reporting numbers, we delay the system a few seconds in order to show how some time calculations will work, and also close our ticket to show that John completed the ticket in 5 seconds.  Note that completion time is unique to Tickets, which our report will utilize"

(Delay forSeconds: 5.1) wait.
(techTickApp getMatchActivityById: newTicketId) closeActivity. "Close the ticket"
self assert: (techTickApp getMatchActivityById: newTicketId) isClosed.

"Our example provided shows a subclass of MatcherReport which includes a report specific to the Tech/Ticket system.  It will use the Ticket-specific field of 'completionTime' in order to show us how long techs are taking to resolve their tickets for all time.  You can subclass this to include any reports on items you may need to make your system more useful."

newReportDict := TechReport reportTicketCompletionTimesByTech.

"Now we can see that John Smith (who was assigned to our ticket) took 5 seconds to complete the ticket"

self assert: (newReportDict at: (techTickApp getMatchResourceById: newTechId)) seconds = 5.

"Notes On Other Classes:
- MatcherScore is the object provided when utilizing Activity/Resource matching algorithms and is a handy way to store those values (out of 100) along with the resource that they matched on.
- MatcherUser is utilized for logins and keeping track of who has created or updated items in the system" ! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:32'!
testMatchBestWorkerByScore
"Second worker has 0 score in the skills match"
| worker addedTicketNum resultingWorkers |
"see setUp for Pepe and someProject"
worker := Worker withFirstName: 'Juanito' lastName: 'Augenblick' address: '123 Main Street' city: 'Shiny Town' state: 'TX' zip: '12333' phone: '5555555454'. 
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'HTML').
workerMatchApp addMatchResource: worker.

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

Transcript show: (someProject bestMatchDictionary).  Transcript cr.

self assert: resultingWorkers size = 2.

self assert: (someProject matchWithTopScore) getResource fullName = 'Pepe Skillz'.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/25/2013 23:25'!
testMatchBestWorkerForProjectOutOfPoolOfWorkers
"Second worker has 0 score in the skills match"
| resultingWorkers |

workerMatchApp addMatchResource:  (Worker withFirstName: 'Juanito' lastName: 'Augenblick' address: '123 Main Street' city: 'Shiny Town' state: 'TX' zip: '12333' phone: '5555555454').

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

self assert: resultingWorkers size = 1.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:37'!
testMatchProjectFindNoWorker
| resultingWorkers |

"Worker Creation"
workerMatchApp addMatchResource:  (Worker withFirstName: 'PepeNo' lastName: 'Skillz' address: '0 Worker Way' city: 'Worker Town' state: 'WA' zip: '22333' phone: '5555551414' ).

workerMatchApp removeMatchResource: (workerMatchApp getMatchResourceById: pepeId).

resultingWorkers := workerMatchApp findResourceMatchForActivity:  someProject.

self assert: resultingWorkers size = 0.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 21:33'!
testMatchProjectWithWorker
| resultingWorkers |

resultingWorkers := workerMatchApp findResourceMatchForActivity: someProject.

self assert: resultingWorkers size = 1.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 21:27'!
testMatchTicketWithTech
| resultingTechs |

resultingTechs := techTicketApp findResourceMatchForActivity: someTicket.

self assert: resultingTechs size = 1.

self assert: (someTicket matchWithTopScore) getResource fullName = 'Test Techer'.

! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:38'!
testMatchingScoreAvailableAt

self assert: ((techTicketApp getMatchResourceById: testTechId) isAvailableAt: #(wednesday friday)) .
self deny: ((techTicketApp getMatchResourceById: testTechId) isAvailableAt: #(thursday friday)) .
self deny: ((techTicketApp getMatchResourceById: testTechId) isAvailableAt: #(sunday thursday friday)) .
self assert: ((techTicketApp getMatchResourceById: testTechId) isAvailableAt: #(monday)) .
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:38'!
testMatchingScoreSkills

self assert: 57.14 = (someProject matchScore: (workerMatchApp getMatchResourceById: pepeId)) getScore. 
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:08'!
testPersonCreation

| person |

person := MatcherResource withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.

self assert: 'Test Techer'  = person fullName.
self assert: '1 Tech Way, Tech City, PA 19191'  = person fullAddress.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 22:03'!
testTechCompletionTimeReport
	"Creates a dictionary of average times it took for a tech to be assigned to its completion time."
	"The MatchReport class has been subclassed here to define the report on all MatchResources in the system."
	"Reports that do not require any subclass (for unique instance items, for example) could be utilized in the"
	"MatcherReport superclass and then called directly, but may be better off in the subclass for readability."
	| avgTimeDict |
	(Delay forSeconds: 2.1) wait.
	(techTicketApp getMatchActivityById: someTicketId) closeActivity.
	(Delay forSeconds: 1.1) wait.
	(techTicketApp getMatchActivityById: secondTicketId) closeActivity.
	avgTimeDict := TechReport reportTicketCompletionTimesByTech.
	Transcript show: avgTimeDict.
	self assert: (avgTimeDict at: (techTicketApp getMatchResourceById: billId)) seconds = 2.
	self assert: (avgTimeDict at: (techTicketApp getMatchResourceById: emptyTechId)) seconds = 3.! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:02'!
testTechCreation
| tech |

tech := (Tech withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Philadelphia').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Washington DC').
tech assignMatchItem: (techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 4).
tech assignMatchItem: (techTicketApp getMatchItemByName: 'Testing').
tech assignMatchItem: (techTicketApp getMatchItemByName: 'CCNA').
tech setAvailability: #(monday wednesday friday) asOrderedCollection.

self assert: true = (tech hasMatchItem: (techTicketApp getMatchItemByName: 'CCNA')).

self assert: (tech hasMatchItem: ((techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 1))).
self deny: (tech hasMatchItem: ((techTicketApp getMatchSkillByName: 'Computer Repair' withSkill: 9))).
! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 21:23'!
testTicketCreation
| newTicket addedTicketNum |

newTicket := Ticket newWithDescription: 'Test Ticket' priority: 1 dueBy: (DateAndTime now) createdBy: (MatcherUser fullName: 'The Creator').

newTicket addRequirement: (techTicketApp getMatchItemByName: 'Computer Repair').
newTicket addRequirement: (techTicketApp getMatchItemByName: 'New York City').

addedTicketNum := techTicketApp addMatchActivity: (newTicket).

self assert: addedTicketNum =3! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:45'!
testWorkerCreation

| worker daysAvailable |

daysAvailable := #(monday wednesday friday) asOrderedCollection.

"I removed 'traits' from this test... if we want Traits, go ahead and make a MatcherItem subclass and it should be an easy add'"

worker := Worker withFirstName: 'Test' lastName: 'Techer' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'Java').
worker assignMatchItem: (workerMatchApp getMatchItemByName: 'New York City').

self assert: 'Test Techer'  = worker fullName.
self assert: '1 Tech Way, Tech City, PA 19191'  = worker fullAddress.
self assert: (worker hasMatchItem: (workerMatchApp getMatchItemByName: 'Java')). 
self assert: (worker hasMatchItem: (workerMatchApp getMatchItemByName: 'New York City')).! !

!MatcherTest methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 22:48'!
testWorkerMatchApp
| workerOne workerTwo countBeforeAdding |

workerOne := Worker withFirstName: 'John' lastName: 'Rodriguez' address: '123 Mainstreet' city: 'Sunrise' state: 'FL' zip: '33326' phone: '3063933939'.

workerTwo := Worker withFirstName: 'Pepe' lastName: 'Smith' address: '1 Tech Way' city: 'Tech City' state: 'PA' zip: '19191' phone: '5555551212'.

countBeforeAdding:= workerMatchApp resourceCount.

workerMatchApp addMatchResource: workerOne.
workerMatchApp addMatchResource: workerTwo.
self assert:  (workerMatchApp resourceCount) = (countBeforeAdding+2).! !

Object subclass: #MatcherActivity
	instanceVariableNames: 'id requirements description status creator priority dueByTime assignedResourcesDictionary creationDate scoreEngineStrategy lastFetch bestMatchDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherActivity class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:39'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user

^ self new newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user

! !

!MatcherActivity methodsFor: 'adding' stamp: 'HectorGonzalez 4/25/2013 23:08'!
addBestMatchRun: aMatchDictionary
	"The idea here is to keep track of matches for any given item, i.e. a ticket has a matcher algorithm run on it (see it in TechTicketApp) which then"
	"creates a dictionary of best matches with the mapping Tech->MatcherScore.  This dictionary is then a member of the matchDictionary variable"
	"in this class, which has keys which are the dates they were added/generated so we can see progress over time."
	lastFetch := DateAndTime now.
	bestMatchDictionary at: (lastFetch) put: aMatchDictionary ! !

!MatcherActivity methodsFor: 'adding' stamp: 'MattF 4/19/2013 22:41'!
addRequirement: aMatcherItem
	requirements add: aMatcherItem.! !

!MatcherActivity methodsFor: 'adding' stamp: 'MattF 4/19/2013 22:47'!
addRequirementList: aMatcherItemOrdList
	aMatcherItemOrdList do: [:ele | requirements add: ele].! !

!MatcherActivity methodsFor: 'assignment' stamp: 'MattF 4/26/2013 23:50'!
assignResourceById: aMatcherResourceId
	assignedResourcesDictionary at: aMatcherResourceId put: (DateAndTime now).! !

!MatcherActivity methodsFor: 'accessing' stamp: 'MattF 4/21/2013 15:10'!
assignedResourcesDictionary
	^ assignedResourcesDictionary! !

!MatcherActivity methodsFor: 'accessing' stamp: 'Mattf 4/13/2013 22:39'!
bestMatchDictionary
	^ bestMatchDictionary! !

!MatcherActivity methodsFor: 'status' stamp: 'MattF 4/26/2013 22:38'!
closeActivity
	status := MatcherStatus Closed.! !

!MatcherActivity methodsFor: 'accessing' stamp: 'MattF 4/16/2013 22:02'!
creationDate
	^ creationDate! !

!MatcherActivity methodsFor: 'accessing' stamp: 'Mattf 4/13/2013 22:41'!
id
	^ id.! !

!MatcherActivity methodsFor: 'status' stamp: 'HectorGonzalez 4/16/2013 22:16'!
isClosed
	^ (status = MatcherStatus Closed)! !

!MatcherActivity methodsFor: 'status' stamp: 'HectorGonzalez 4/16/2013 22:16'!
isOpen
	^ (status = MatcherStatus Open)! !

!MatcherActivity methodsFor: 'matching' stamp: 'MattF 4/26/2013 23:47'!
matchScore: aMatcherResource
	^scoreEngineStrategy matchScoreForResource: aMatcherResource  withActivity: self.! !

!MatcherActivity methodsFor: 'matching' stamp: 'MattF 4/26/2013 23:45'!
matchWithTopScore
	^scoreEngineStrategy matchWithTopScore: bestMatchDictionary lastFetch: lastFetch.! !

!MatcherActivity methodsFor: 'initialize-release' stamp: 'MattF 4/26/2013 23:45'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user


description := desc.
priority := priorityNum.
dueByTime := dueByDate.
status := MatcherStatus Open. 
creator := user.
assignedResourcesDictionary := Dictionary new.
requirements := OrderedCollection new.
bestMatchDictionary := Dictionary new.
creationDate := (DateAndTime now).
scoreEngineStrategy := MatcherScoreEngine new.
! !

!MatcherActivity methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 23:11'!
requirements
	^requirements! !

!MatcherActivity methodsFor: 'accessing' stamp: 'MattF 4/27/2013 23:15'!
scoreEngineStrategy: aScoreEngineStrategy
	scoreEngineStrategy := aScoreEngineStrategy.! !

MatcherActivity subclass: #Project
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Project methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:55'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user

id := WorkerMatchApp default nextActivityId.
super newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user.! !

MatcherActivity subclass: #Ticket
	instanceVariableNames: 'completionTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Ticket methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 21:17'!
closeActivity
	completionTime := (DateAndTime now).
	super closeActivity.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 21:42'!
completionTime
	^ completionTime.! !

!Ticket methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 21:19'!
newWithDescription: desc priority: priorityNum dueBy: dueByDate  createdBy: user

id := TechTicketApp default nextActivityId.
completionTime := 0.
super newWithDescription: desc priority: priorityNum dueBy: dueByDate createdBy: user.! !

Object subclass: #MatcherApp
	instanceVariableNames: 'masterMatchItemCollection activityCollection resourceCollection scoreEngine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherApp class
	instanceVariableNames: 'soleInstance'!

MatcherApp class
	instanceVariableNames: 'soleInstance'!

!MatcherApp methodsFor: 'accessing' stamp: 'MattF 4/24/2013 21:40'!
activityCollection
	^ activityCollection! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:27'!
activityCount
	^ activityCollection size.! !

!MatcherApp methodsFor: 'adding' stamp: 'MattF 4/19/2013 23:30'!
addMatchActivity: aMatcherActivity
	activityCollection add: aMatcherActivity.
	^ aMatcherActivity id! !

!MatcherApp methodsFor: 'adding' stamp: 'MattF 4/20/2013 15:50'!
addMatchItem: aMatcherItem
	masterMatchItemCollection add: aMatcherItem.
! !

!MatcherApp methodsFor: 'adding' stamp: 'MattF 4/24/2013 22:31'!
addMatchResource: aMatcherResource
	resourceCollection add: aMatcherResource.
	^ aMatcherResource id.! !

!MatcherApp methodsFor: 'converting' stamp: 'MattF 4/24/2013 23:06'!
assignResourceId: aResourceId toActivityId: anActivityId
	activityCollection do: [:ele | (ele id = anActivityId) ifTrue: [ele assignResourceById: aResourceId. ^ self.]].! !

!MatcherApp methodsFor: 'initialize-release' stamp: 'HectorGonzalez 4/25/2013 22:44'!
buildInitialSet
"Implement this to add some default values for logical items and initialize the others to a base collection"
	self subclassResponsibility! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 23:40'!
findResourceMatchForActivity: aMatcherActivity
	"Provides a dictionary of scores, with resources as keys for a given activity"
	| matches checkScore |
	
	matches := (scoreEngine getResourcesScores: resourceCollection  forActivity: aMatcherActivity).
	
	aMatcherActivity addBestMatchRun: matches.	
		
	^ matches.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:06'!
getMatchActivityById: anIdNumber
	"Returns a ticket based on its ID number"
	| aTicket |
	activityCollection do: [:ele | ((ele id) = anIdNumber) ifTrue: [ ^ ele ] ].
	^ nil.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:07'!
getMatchItemByName: aName
	"Returns match items in the system searching by name"
	masterMatchItemCollection do: [:ele | (ele getName = aName) 
		ifTrue: [^ ele copy]
		].
	self error: 'Item not found (', aName, '), add it to the system first'..! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:36'!
getMatchResourceById: anIdNumber
	"Returns a MatchResource based on it's ID number"
	resourceCollection do: [:ele | ((ele id) = anIdNumber) ifTrue: [ ^ ele ] ].
	^ nil.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:07'!
getMatchSkillByName: aName withSkill: aNumber
	"Provides a MatcherItem with the given skill name and level"
	| tempSkill |
	masterMatchItemCollection do: [:ele | (ele getName = aName & (ele class asString) = 'MatcherSkill') 
		ifTrue: [tempSkill := ele copy.  tempSkill level: aNumber. ^ tempSkill.]
		].
	self error: 'Item not found (', aName, '), add it to the system first'..! !

!MatcherApp methodsFor: 'initialize-release' stamp: 'HectorGonzalez 4/25/2013 23:36'!
initialize
	masterMatchItemCollection := OrderedCollection  new.
	activityCollection := OrderedCollection new.
	resourceCollection := OrderedCollection  new.
	scoreEngine := MatcherScoreEngine new.
	self buildInitialSet.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:43'!
nextActivityId
"Provides the next activity id from the system, so we don't ever re-use an ID"
	| newActivityNum |
	newActivityNum := 1.
	(activityCollection size = 0) ifTrue:[^newActivityNum].
	activityCollection do: [:element | 
		((element = nil) or: (element id) = nil) ifFalse: [ 
			
							((element id) >= newActivityNum)
							 ifTrue: [newActivityNum := (element id) +1]
			
								].
	].
	^ newActivityNum.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:07'!
nextResourceId
"Provides the next resource id from the system, so we don't ever re-use an ID"
	| newResourceNum |
	newResourceNum := 1.
	(resourceCollection size = 0) ifTrue:[^newResourceNum].
	resourceCollection do: [:element | 
		((element = nil) or: (element id) = nil) ifFalse: [ 
			
							((element id) >= newResourceNum)
							 ifTrue: [newResourceNum := (element id) +1]
			
								].
	].
	^ newResourceNum.! !

!MatcherApp methodsFor: 'private' stamp: 'MattF 4/20/2013 01:24'!
privateClearItems
	self initialize.! !

!MatcherApp methodsFor: 'removing' stamp: 'MattF 4/20/2013 15:49'!
removeMatchActivity: aMatcherActivity
	activityCollection remove: aMatcherActivity.
	^ aMatcherActivity id! !

!MatcherApp methodsFor: 'removing' stamp: 'MattF 4/20/2013 15:51'!
removeMatchItem: aMatcherItem
	masterMatchItemCollection remove: aMatcherItem.
! !

!MatcherApp methodsFor: 'removing' stamp: 'MattF 4/20/2013 15:49'!
removeMatchResource: aMatcherResource
	resourceCollection remove: aMatcherResource.! !

!MatcherApp methodsFor: 'accessing' stamp: 'MattF 4/21/2013 14:51'!
resourceCollection 
	^ resourceCollection.! !

!MatcherApp methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:27'!
resourceCount
	^ resourceCollection size.! !

MatcherApp subclass: #TechTicketApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!TechTicketApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:31'!
default
	(soleInstance isNil) ifTrue: [soleInstance := super new initialize].
	^ soleInstance.! !

!TechTicketApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:34'!
new
	self error: 'This is a singleton, use default'! !

!TechTicketApp methodsFor: 'initialize-release' stamp: 'HectorGonzalez 4/25/2013 22:42'!
buildInitialSet
	 
"there should be some default locations I would think...user can take them out if not needed?"
masterMatchItemCollection add: (MatcherLocation newWithName: 'New York City' address: '1 Main St.' city: 'New York' state: 'NY' additionalInfo: 'Main NY Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Washington DC' address: '1600 Pennsylvania Ave.' city: 'Washington' state: 'DC' additionalInfo: 'Main DC Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'San Francisco' address: '100 Lombard St.' city: 'San Francisco' state: 'CA' additionalInfo: 'Main SF Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Orlando' address: '876 Disney Pl.' city: 'Orlando' state: 'FL' additionalInfo: 'Main FL Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'Philadelphia' address: '2 Bell Blvd.' city: 'Philadelphia' state: 'PA' additionalInfo: 'Main PHL Office').

"adding in some base skills... no point in adding a level, that should be user set for either the ticket to assign it to or the tech to assign it to... this will just allow us a base list to show"
masterMatchItemCollection add: (MatcherSkill withName: 'Computer Repair' level: 0 description: 'Ability to fix computers').
masterMatchItemCollection add: (MatcherSkill withName: 'Electrical Repair' level: 0 description: 'Electrical Certification').
masterMatchItemCollection add: (MatcherSkill withName: 'Testing' level: 0 description: 'Apitude as a tester').
masterMatchItemCollection add: (MatcherSkill withName: 'Java' level: 0 description: 'Apitude in Java').
masterMatchItemCollection add: (MatcherSkill withName: 'Teaching' level: 0 description: 'Ability to teach').

masterMatchItemCollection add: (MatcherCertification withName: 'CCNA' andDescription: 'ccna cert').! !

MatcherApp subclass: #WorkerMatchApp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!WorkerMatchApp class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:30'!
default
	(soleInstance isNil) ifTrue: [soleInstance := super new initialize].
	^ soleInstance.! !

!WorkerMatchApp class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 11:34'!
new
	self error: 'This is a singleton, use default'! !

!WorkerMatchApp methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 22:43'!
buildInitialSet
"Some skills to begin this application with"
masterMatchItemCollection add: (MatcherSkill withName: 'HTML' level: 0 description: 'Ability to write HTML').
masterMatchItemCollection add: (MatcherSkill withName: 'Java' level: 0 description: 'Ability to write Java').
masterMatchItemCollection add: (MatcherSkill withName: 'CSS' level: 0 description: 'Ability to write CSS').
masterMatchItemCollection add: (MatcherSkill withName: 'PHP' level: 0 description: 'Ability to write PHP').
masterMatchItemCollection add: (MatcherSkill withName: 'Coding' level: 0 description: 'Ability to Code').

"Some common locations to begin this application with"
masterMatchItemCollection add: (MatcherLocation newWithName: 'New York City' address: '1 Main St.' city: 'New York' state: 'NY' additionalInfo: 'Main NY Office').
masterMatchItemCollection add: (MatcherLocation newWithName: 'ECommerce' address: 'N/A' city: 'N/A' state: '' additionalInfo: 'Online Store').! !

Object subclass: #MatcherItem
	instanceVariableNames: 'name description level createdDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherItem class
	instanceVariableNames: 'weight'!

MatcherItem class
	instanceVariableNames: 'weight'!

MatcherItem subclass: #MatcherCertification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:41'!
setWeight: aNumber
	(aNumber > 1 | aNumber < 0) ifTrue: [self error: 'Weights should be between zero and one.'] ifFalse: [weight := aNumber].! !

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:47'!
weight
	^ weight.! !

!MatcherItem class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 16:46'!
withName: aName andDescription:aDescription
	^ self new withName: aName andDescription: aDescription.! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 17:18'!
= aMatcherItem
	(self class) = (aMatcherItem class) ifFalse: [^ false].
	(self getName = aMatcherItem getName) ifTrue: [^ true].
	^ false.! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:04'!
getDescription
^description! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:04'!
getName
^name! !

!MatcherItem methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 17:42'!
withName: aName andDescription:aDescription
name:= aName.
description:=aDescription.
createdDate := (DateAndTime now).! !

MatcherItem subclass: #MatcherLocation
	instanceVariableNames: 'address city state additionalInfo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherLocation class methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 01:02'!
empty

^ (MatcherLocation new) newWithName: 'Unknown' address: '' city: '' state: '' additionalInfo: 'Location has not been set'! !

!MatcherLocation class methodsFor: 'as yet unclassified' stamp: 'MattF 3/31/2013 13:04'!
newWithName: strName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo

^ self new newWithName: strName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 17:18'!
= aMatcherLocation
	(self class) = (aMatcherLocation class) ifFalse: [^ false].
	(self address = aMatcherLocation address & self city = aMatcherLocation city & self state = aMatcherLocation state) ifTrue: [^ true].
	^ false.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:52'!
address
	^ address! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:53'!
city
	^ city! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:18'!
newWithName: aName address: strAddress city: strCity state: strState additionalInfo: strAdditionalInfo
address := strAddress.
city := strCity.
state := strState.
additionalInfo := strAdditionalInfo.

super withName:aName andDescription:''.! !

!MatcherLocation methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 21:53'!
state
	^ state! !

MatcherItem subclass: #MatcherSkill
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherSkill class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/7/2013 22:32'!
withName: strName level: numLevel description: strDesc

^ self new withName: strName level: numLevel description: strDesc.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 23:18'!
= aMatcherSkill
"(self getName = 'Computer Repair') ifTrue: [self halt]."
	(self class) = (aMatcherSkill class) ifFalse: [^ false].
	(aMatcherSkill getName = self getName) & (self @ aMatcherSkill) ifTrue: [^ true].
	^ false.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:03'!
@ aMatcherSkill
	"Equals is defined in MatcherItem since that can just be name... but this shorthand should help us"
	"tell if a given skill is at or higher than the skill passed to it, hence use of @"
	(self level >= aMatcherSkill level) ifTrue: [^ true].
	^ false.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 22:34'!
level
	^ level! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'Mattf 4/10/2013 23:22'!
level: aNumber
	level := aNumber.! !

!MatcherSkill methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:17'!
withName: aName level: numLevel description: aDescription
level := numLevel .
super withName:aName andDescription: aDescription.! !

MatcherItem subclass: #MatcherStatus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 20:58'!
Closed
	^ self new newWithName: 'Closed'.! !

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 20:58'!
Open
	^ self new newWithName: 'Open'.! !

!MatcherStatus class methodsFor: 'as yet unclassified' stamp: 'Mattf 4/7/2013 20:59'!
newWithName: aName
	^ self new newWithName: aName.! !

!MatcherStatus methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:19'!
newWithName: aName
	"I called this strName because name was overriding some base method in Object, which didn't seem right to do"
super withName:aName andDescription: ''.! !

Object subclass: #MatcherReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherReport class methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 18:03'!
reportTicketVolumeByHour: aMatcherApp
	| resultsColl |
	resultsColl := OrderedCollection newFrom: (0 to: 24 by: 0.5).
	resultsColl := Dictionary newFromPairs: resultsColl.
	resultsColl keysDo: [:ele | resultsColl at: ele put: 0].
	
	aMatcherApp activityCollection do: [:ele | resultsColl at: ((ele creationDate) hour24) put: ((resultsColl at: ((ele creationDate) hour24))+1)].
	^ resultsColl 
	
	! !

MatcherReport subclass: #TechReport
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!TechReport class methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 18:13'!
reportTicketCompletionTimesByTech
	"This report will provide the average duration a tech took to complete a ticket item."
	| techDict techTicketCounts |
	techDict := Dictionary new.
	techTicketCounts := Dictionary new.
	TechTicketApp default resourceCollection do: [:ele | techDict at: ele put: (Duration new).  techTicketCounts at: ele put: 0.].
	
	TechTicketApp default activityCollection do: [:ele | (ele isClosed) ifTrue: [
			ele assignedResourcesDictionary keysDo: [
				:aRes | techDict at: (TechTicketApp default getMatchResourceById: aRes) 
								put: ((techDict at: (TechTicketApp default getMatchResourceById: aRes)) + ((ele completionTime) - (ele assignedResourcesDictionary at: aRes))).
					     techTicketCounts at: (TechTicketApp default getMatchResourceById: aRes) put: ((techTicketCounts at: (TechTicketApp default getMatchResourceById: aRes)) + 1).].
			]].
	
	techTicketCounts keysDo: [:ele | ((techTicketCounts at: ele) = 0) ifFalse: [techDict at: ele put: ((techDict at: ele) / (techTicketCounts at: ele))]].
	
	^ techDict 
! !

Object subclass: #MatcherResource
	instanceVariableNames: 'id matchItemList fname lname address city state zip phone description daysAvailable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherResource class methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:34'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone

^ self new withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:11'!
assignMatchItem: aMatcherItem
	matchItemList add: aMatcherItem.! !

!MatcherResource methodsFor: 'accessing' stamp: 'HectorGonzalez 3/31/2013 20:57'!
fullAddress
	^ (address, ', ', city, ', ', state, ' ', zip)! !

!MatcherResource methodsFor: 'accessing' stamp: 'HectorGonzalez 3/31/2013 20:48'!
fullName
  ^(fname, ' ', lname)! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/19/2013 23:00'!
hasMatchItem: aMatchItem
	^(matchItemList includes: aMatchItem).! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 14:23'!
id
	^ id.! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 21:54'!
isAvailableAt: theseDays
	"Didnt think this needed a score, just a true or false really..."
	theseDays do: [:ele | (daysAvailable includes: ele) ifFalse: [^ false]].
	^ true.
	
	"^(daysAvailable getMatchScore:  theseDays forResource: self)"! !

!MatcherResource methodsFor: 'as yet unclassified' stamp: 'MattF 4/20/2013 00:07'!
matchItemList
	^ matchItemList ! !

!MatcherResource methodsFor: 'testing' stamp: 'Mattf 4/13/2013 22:44'!
printOn: aStream
	self fullName printOn: aStream.! !

!MatcherResource methodsFor: 'testing' stamp: 'MattF 4/20/2013 21:35'!
setAvailability: aCollection
	daysAvailable addAll: aCollection.! !

!MatcherResource methodsFor: 'initialize-release' stamp: 'MattF 4/24/2013 23:00'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone
	

fname := strFName.
lname := strLName.
address := strAddress.
city := strCity.
state := strState.
zip := strZip.
phone := strPhone.
matchItemList := OrderedCollection new.

daysAvailable := OrderedCollection new.! !

MatcherResource subclass: #Tech
	instanceVariableNames: 'currentLocation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Tech methodsFor: 'as yet unclassified' stamp: 'MattF 4/24/2013 22:31'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone

id := TechTicketApp default nextResourceId.

super 		withFirstName: strFName 
			lastName: strLName 
			address: strAddress 
			city: strCity 
			state: strState 
			zip: strZip 
			phone: strPhone.

currentLocation := nil.
! !

MatcherResource subclass: #Worker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!Worker class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/1/2013 00:09'!
withSkillsCollection: skillsArr locationCollection: locationArr firstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone traits: traitsArr daysAvailable: aSetOfDaysAvailable

^ self new withSkillsCollection: skillsArr locationCollection: locationArr firstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone traits: traitsArr daysAvailable: aSetOfDaysAvailable.

! !

!Worker methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:22'!
withFirstName: strFName lastName: strLName address: strAddress city: strCity state: strState zip: strZip phone: strPhone

id := WorkerMatchApp default nextResourceId.

super 		withFirstName: strFName 
			lastName: strLName 
			address: strAddress 
			city: strCity 
			state: strState 
			zip: strZip 
			phone: strPhone.
! !

Object subclass: #MatcherScore
	instanceVariableNames: 'score fullMatch resource'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

MatcherScore class
	instanceVariableNames: 'scaleDictionary'!

MatcherScore class
	instanceVariableNames: 'scaleDictionary'!

!MatcherScore class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/19/2013 18:02'!
newWithScore: aNumber andResource:aResource
	^ self new newWithScore: aNumber andResource: aResource.! !

!MatcherScore methodsFor: 'accessing' stamp: 'HectorGonzalez 4/19/2013 18:01'!
getResource
	^resource! !

!MatcherScore methodsFor: 'accessing' stamp: 'HectorGonzalez 4/8/2013 10:43'!
getScore
	^score! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/8/2013 13:33'!
isFullMatch
	^(score = 100).! !

!MatcherScore methodsFor: 'initialize-release' stamp: 'MattF 4/27/2013 00:17'!
newWithScore: aNumber
	"Provides an object for storing numeric scores that generally represent the relationship between a resource and activity"
	"based on the strategy used"
	score := aNumber roundTo: 0.01.! !

!MatcherScore methodsFor: 'initialize-release' stamp: 'MattF 4/20/2013 23:31'!
newWithScore: aNumber andResource: aResource
	score := aNumber roundTo: 0.01.
	resource := aResource.! !

!MatcherScore methodsFor: 'as yet unclassified' stamp: 'Mattf 4/13/2013 22:43'!
printOn: aStream
	score printOn: aStream.! !

Object subclass: #MatcherScoreEngine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherScoreEngine methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 23:40'!
getResourcesScores: resourceCollection forActivity: aMatcherActivity
	| matchesDict checkScore |
	matchesDict  := Dictionary new.
	resourceCollection do: [:techEle | 
		checkScore := (aMatcherActivity matchScore: techEle).  
		(checkScore getScore) = 0 ifFalse: [matchesDict at: techEle put: checkScore ]].
	^matchesDict.
	
	
! !

!MatcherScoreEngine methodsFor: 'as yet unclassified' stamp: 'MattF 4/26/2013 23:49'!
matchScoreForResource: aMatcherResource withActivity: aMatchActivity
"accepts a matcher resource object to provide a score for how well they match up, based on the weightings set on your MatchItems"
| tempScore tempTotalWeight |
"note:  I calculate the total weight based on what we see in the MatcherItem classes, so it may not be out of 100"
"but we will end up with a normal percent to keep things normal"
tempScore := 0.
tempTotalWeight := 0.
(aMatchActivity requirements size) > 0 
	ifTrue: [aMatchActivity requirements do: 
				[:ele | (aMatcherResource hasMatchItem: ele) ifTrue: [tempScore := (tempScore asFloat) + (ele class weight)].  
					tempTotalWeight := tempTotalWeight + ele class weight.]. 
				tempScore :=  (tempScore / tempTotalWeight) * 100.] 
	ifFalse: [tempScore := 100]. 

"The weighting is now set in the class variable weight by MatcherItem type, which could be set up by the user or just on app startup"
"This allows a dynamic list of requirements objects without us having to know what type they are but still get a weighted score"

^ MatcherScore newWithScore: tempScore andResource: aMatcherResource.! !

!MatcherScoreEngine methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/25/2013 23:08'!
matchWithTopScore: bestMatchDictionary lastFetch: aDateTime
|topResource|

(bestMatchDictionary at:aDateTime) do: [:resource|	
		(topResource = nil)
			ifTrue:[
				topResource := resource.
			].		
		(resource getScore) >  (topResource getScore)
			ifTrue:[
				topResource := resource.
			].
	].

^ topResource.! !

Object subclass: #MatcherUser
	instanceVariableNames: 'fullName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Matcher'!

!MatcherUser class methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/7/2013 17:48'!
fullName: aFullName
	^self new fullName: aFullName! !

!MatcherUser methodsFor: 'as yet unclassified' stamp: 'HectorGonzalez 4/7/2013 17:49'!
fullName: aFullName
fullName:=aFullName! !

!MatcherUser methodsFor: 'as yet unclassified' stamp: 'MattF 4/27/2013 00:19'!
initialize
	"MatcherUser is used to track who makes changes in the system and for possible login setups"
	fullName := ''.
	super initialize.! !
